package org.anddev.andengine.util.pool;

import java.util.Stack;

import org.anddev.andengine.util.Debug;

/**
 * @author Valentin Milea
 * @author Nicolas Gramlich
 * 
 * @since 23:00:21 - 21.08.2010
 * @param <T>
 */
abstract class Pool<T extends PoolItem> {
	// ===========================================================
	// Constants
	// ===========================================================

	// ===========================================================
	// Fields
	// ===========================================================

	private final Stack<T> mAvailableItems = new Stack<T>();
	private int mUnrecycledCount;

	// ===========================================================
	// Constructors
	// ===========================================================

	public Pool(final int pInitialSize) {
		for(int i = pInitialSize - 1; i >= 0; i--) {
			this.mAvailableItems.push(this.allocate());
		}
	}

	// ===========================================================
	// Getter & Setter
	// ===========================================================

	// ===========================================================
	// Methods for/from SuperClass/Interfaces
	// ===========================================================

	protected abstract T allocate();

	// ===========================================================
	// Methods
	// ===========================================================

	public synchronized int getUnrecycledCount() {
		return this.mUnrecycledCount;
	}

	public synchronized T obtain() {
		T item;

		if(this.mAvailableItems.size() > 0) {
			item = this.mAvailableItems.pop();
		} else {
			Debug.d(this.getClass().getSimpleName() + " Pool exhausted, with " + this.mUnrecycledCount + " unrecycled items. Allocating more...");
			item = this.allocate();
		}

		// assert: item.mParent == null
		item.mParent = this;
		this.mUnrecycledCount++;
		return item;
	}

	public synchronized void recyle(final T pItem) {
		if(pItem.mParent != this) {
			if(pItem.mParent == null) {
				throw new IllegalArgumentException("Item already recycled. Maybe it's from another pool?");
			} else {
				throw new IllegalArgumentException("Item from another pool!");
			}
		}

		pItem.mParent = null;
		this.mAvailableItems.push(pItem);

		// assert: mUnrecycledCount >= 0
		this.mUnrecycledCount--;
	}

	public synchronized boolean ownsItem(final T pItem) {
		return pItem.mParent == this;
	}

	@SuppressWarnings("unchecked")
	void recyclePoolItem(final PoolItem pItem) {// TODO Why not "T" ?
		this.recyle((T) pItem);
	}

	// ===========================================================
	// Inner and Anonymous Classes
	// ===========================================================
}
