package org.anddev.andengine.util.pool;

import java.util.ArrayList;

import org.anddev.andengine.engine.handler.IUpdateHandler;

/**
 * @author Valentin Milea
 * @author Nicolas Gramlich
 * 
 * @since 23:02:58 - 21.08.2010
 * @param <T>
 */
abstract class PoolUpdateHandler<T extends PoolItem> implements IUpdateHandler {
	// ===========================================================
	// Constants
	// ===========================================================

	// ===========================================================
	// Fields
	// ===========================================================

	private final Pool<T> mPool;
	private final ArrayList<T> mScheduled = new ArrayList<T>();

	// ===========================================================
	// Constructors
	// ===========================================================

	public PoolUpdateHandler(final int pInitialPoolSize) {
		this.mPool = new Pool<T>(pInitialPoolSize) {
			@Override
			protected T allocate() {
				return PoolUpdateHandler.this.allocate();
			}
		};
	}

	// ===========================================================
	// Getter & Setter
	// ===========================================================

	// ===========================================================
	// Methods for/from SuperClass/Interfaces
	// ===========================================================

	protected abstract T allocate();

	protected abstract void handle(T pItem);

	@Override
	public void onUpdate(final float pSecondsElapsed) {
		final ArrayList<T> scheduled = this.mScheduled;

		synchronized (scheduled) {
			final int count = scheduled.size();

			if(count > 0) {
				final Pool<T> pool = this.mPool;
				T item;

				for(int i = 0; i < count; i++) {
					item = scheduled.get(i);
					this.handle(item);
					pool.recyle(item);
				}

				scheduled.clear();
			}
		}
	}

	@Override
	public void reset() {
		synchronized (this.mScheduled) {
			final int count = this.mScheduled.size();

			for(int i = 0; i < count; i++) {
				this.mPool.recyle(this.mScheduled.get(i));
			}

			this.mScheduled.clear();
		}
	}

	// ===========================================================
	// Methods
	// ===========================================================

	public T obtain() {
		return this.mPool.obtain();
	}

	public void postBack(final T pItem) {
		synchronized (this.mScheduled) {
			if(!this.mPool.ownsItem(pItem)) {
				throw new IllegalArgumentException("Runnable from another pool" + "or already recycled!");
			}

			this.mScheduled.add(pItem);
		}
	}

	// ===========================================================
	// Inner and Anonymous Classes
	// ===========================================================
}
